/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */
import { CommonModule } from "@angular/common";
import { Component } from "@angular/core";
import { ReactiveFormsModule, FormControl } from "@angular/forms";
import { RouterModule } from "@angular/router";
import { ApiService } from "../../api.service";
import { ConsentComponent } from "../../consent/consent.component";
import { ErrorComponent } from "../../error/error.component";
import { CheckboxComponent } from "../../inputs/checkbox/checkbox.component";
import { MultiLineComponent }
  from "../../inputs/multi-line/multi-line.component";
import { SelectMenuComponent }
  from "../../inputs/select-menu/select-menu.component";
import { SingleLineComponent }
  from "../../inputs/single-line/single-line.component";
import { SuccessComponent } from "../../success/success.component";
import { UserinfoComponent } from "../../userinfo/userinfo.component";
import type { Platform } from "@repo/types/prod/unions/platform";
import type { Sanction } from "@repo/types/prod/unions/sanction";

@Component({
  imports: [
    RouterModule,
    CommonModule,
    ConsentComponent,
    UserinfoComponent,
    ErrorComponent,
    ReactiveFormsModule,
    CheckboxComponent,
    SingleLineComponent,
    SelectMenuComponent,
    MultiLineComponent,
    SuccessComponent,
  ],
  selector:    "appeal-form",
  styleUrl:    "./appeal.component.css",
  templateUrl: "./appeal.component.html",
})
export class AppealComponent {
  public loading = false;
  public error = "";
  public success = false;

  public firstName = new FormControl("");
  public lastName = new FormControl("");
  public email = new FormControl("");
  public company = new FormControl("");

  public understandBinding = new FormControl(false);
  public sanctionType = new FormControl("reminder");
  public caseNumber = new FormControl("");
  public sanctionPlatform = new FormControl<Platform>("Forum");
  public platformUsername = new FormControl("");
  public sanctionReason = new FormControl("");
  public sanctionFair = new FormControl("");
  public behaviourViolation = new FormControl("");
  public appealReason = new FormControl("");
  public behaviourImprove = new FormControl("");

  public consent = new FormControl(false);

  public constructor(private readonly apiService: ApiService) {}

  // eslint-disable-next-line complexity -- stfu
  public submit(event: MouseEvent): void {
    this.error = "";
    const { form } = event.target as HTMLButtonElement;
    const valid = form?.reportValidity();
    if (valid !== true) {
      return;
    }
    this.loading = true;

    this.apiService.
      submitAppeal({
        appealReason:       this.appealReason.value ?? undefined,
        behaviourImprove:   this.behaviourImprove.value ?? undefined,
        behaviourViolation: this.behaviourViolation.value ?? undefined,
        caseNumber:         Number.parseInt(this.caseNumber.value ?? "0", 10),
        consent:            this.consent.value ?? false,
        email:              this.email.value ?? undefined,
        firstName:          this.firstName.value ?? undefined,
        lastName:           this.lastName.value ?? undefined,
        platformUsername:   this.platformUsername.value ?? undefined,
        sanctionFair:       this.sanctionFair.value ?? undefined,
        sanctionPlatform:   this.sanctionPlatform.value ?? undefined,
        sanctionReason:     this.sanctionReason.value ?? undefined,
        sanctionType:
          (this.sanctionType.value?.split(/\s+/)[0] as Sanction | undefined)
          ?? undefined,
        understandBinding: this.understandBinding.value ?? undefined,
      }).
      then((response) => {
        if ("error" in response) {
          this.error = response.error;
          this.loading = false;
        } else {
          this.error = "";
          this.success = true;
        }
      }).
      catch(() => {
        this.error = "An error occurred while submitting the form.";
        this.loading = false;
      });
  }
}
