/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */
import { CommonModule } from "@angular/common";
import { Component } from "@angular/core";
import { FormControl, ReactiveFormsModule } from "@angular/forms";
import { RouterModule } from "@angular/router";
import { ApiService } from "../../api.service";
import { ConsentComponent } from "../../consent/consent.component";
import { ErrorComponent } from "../../error/error.component";
import { CheckboxComponent } from "../../inputs/checkbox/checkbox.component";
import { MultiLineComponent }
  from "../../inputs/multi-line/multi-line.component";
import { SelectMenuComponent }
  from "../../inputs/select-menu/select-menu.component";
import { SuccessComponent } from "../../success/success.component";
import { UserinfoComponent } from "../../userinfo/userinfo.component";
import type { SessionLength } from "@repo/types/prod/unions/session";

@Component({
  imports: [
    RouterModule,
    CommonModule,
    ConsentComponent,
    UserinfoComponent,
    ErrorComponent,
    ReactiveFormsModule,
    CheckboxComponent,
    SelectMenuComponent,
    MultiLineComponent,
    SuccessComponent,
  ],
  selector:    "meeting-form",
  styleUrl:    "./meeting.component.css",
  templateUrl: "./meeting.component.html",
})
export class MeetingComponent {
  public loading = false;
  public error = "";
  public success = false;

  public firstName = new FormControl("");
  public lastName = new FormControl("");
  public email = new FormControl("");
  public company = new FormControl("");

  public sessionLength = new FormControl("");
  public sessionGoal = new FormControl("");
  public paymentUnderstanding = new FormControl(false);

  public consent = new FormControl(false);

  public constructor(private readonly apiService: ApiService) {}

  // eslint-disable-next-line complexity -- stfu
  public submit(event: MouseEvent): void {
    this.error = "";
    const { form } = event.target as HTMLButtonElement;
    const valid = form?.reportValidity();
    if (valid !== true) {
      return;
    }
    this.loading = true;

    this.apiService.
      submitMeeting({
        companyName:          this.company.value ?? undefined,
        consent:              this.consent.value ?? false,
        email:                this.email.value ?? undefined,
        firstName:            this.firstName.value ?? undefined,
        lastName:             this.lastName.value ?? undefined,
        paymentUnderstanding: this.paymentUnderstanding.value ?? undefined,
        sessionGoal:          this.sessionGoal.value ?? undefined,
        sessionLength:        Number.parseInt(
          this.sessionLength.value ?? "15", 10,
        ) as SessionLength,
      }).
      then((response) => {
        if ("error" in response) {
          this.error = response.error;
          this.loading = false;
        } else {
          this.error = "";
          this.success = true;
        }
      }).
      catch(() => {
        this.error = "An error occurred while submitting the form.";
        this.loading = false;
      });
  }
}
