/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */
import { CommonModule } from "@angular/common";
import { Component } from "@angular/core";
import { FormControl, ReactiveFormsModule } from "@angular/forms";
import { RouterModule } from "@angular/router";
import { ApiService } from "../../api.service.js";
import { ConsentComponent } from "../../consent/consent.component.js";
import { ErrorComponent } from "../../error/error.component.js";
import { MultiLineComponent }
  from "../../inputs/multi-line/multi-line.component.js";
import { SuccessComponent } from "../../success/success.component.js";
import { UserinfoComponent } from "../../userinfo/userinfo.component.js";

@Component({
  imports: [
    RouterModule,
    CommonModule,
    ConsentComponent,
    UserinfoComponent,
    ErrorComponent,
    ReactiveFormsModule,
    MultiLineComponent,
    SuccessComponent,
  ],
  selector:    "app-testimonial",
  styleUrl:    "./testimonial.component.css",
  templateUrl: "./testimonial.component.html",
})
export class TestimonialComponent {
  public loading = false;
  public error = "";
  public success = false;

  public firstName = new FormControl("");
  public lastName = new FormControl("");
  public email = new FormControl("");
  public company = new FormControl("");

  public content = new FormControl("");

  public consent = new FormControl(false);

  public constructor(private readonly apiService: ApiService) {}

  public submit(event: MouseEvent): void {
    this.error = "";
    const { form } = event.target as HTMLButtonElement;
    const valid = form?.reportValidity();
    if (valid !== true) {
      return;
    }
    this.loading = true;

    this.apiService.
      submitTestimonial({
        consent:   this.consent.value ?? false,
        content:   this.content.value ?? undefined,
        email:     this.email.value ?? undefined,
        firstName: this.firstName.value ?? undefined,
        lastName:  this.lastName.value ?? undefined,
      }).
      then((response) => {
        if ("error" in response) {
          this.error = response.error;
          this.loading = false;
        } else {
          this.error = "";
          this.success = true;
        }
      }).
      catch(() => {
        this.error = "An error occurred while submitting the form.";
        this.loading = false;
      });
  }
}
